'use strict'

const { program } = require('commander');
const fs = require('fs')
const diffParse = require('parse-diff')
const cobertura = require( "@cvrg-report/cobertura-json" );

const app = {
  run() {
    program
      .version('0.0.8')
      .description('Compare cobertura.xml generated by jest with a diff patch file')
      .option('-c, --cobertura <cobertura>', 'The file generated by the (feature) branch you want to check', 'coverage/cobertura.xml')
      .option('-d, --diff <diff>', 'The diff patch file ', 'diff.patch')
      .option('-p, --project <project>', 'The project path form the workspace root', '')
      .option('-t, --threshold <threshold>','The minimal threshold code should have', '0.70')
      .option('-v, --variance <variance>','Allow for variance, e.g. allow 0.05 to allow a 5% decrease', '0');
    program.parse(process.argv);
    const options = program.opts();

    console.info(`##Cobertura Diff Coverage ## Version: ${options['version']}`);

    this.validateOptions(options);

    // this example reads the file synchronously
    // you can read it asynchronously also
    let xmlString = fs.readFileSync(options.cobertura, "utf8");
    let diffstring = fs.readFileSync(options.diff, "utf8");

    var diffFile = diffParse(diffstring);

    let exitCode = 0;
    const minCov = parseFloat(options.threshold*100);

    if (options.variance!='0') {
      console.log(`Allowed variance is ${options.variance}`);
    }

    //Parse the cobertura.xml file content to loop through the relevant files.
    cobertura.parseContent(xmlString)
    .then(function (result) {
      var linesToCover = 0
      var coveredLines = 0
      result.forEach(file => {
        let fileLines = file.lines.details
        var fileLinesToCover = 0
        var fileCoveredLines = 0
        //Locate the file chunks into the patch file
        let patchObject = diffFile.find(patch => patch.to === options.project+file.file)
        //Flat the changes form the chunks and filter the additions
        let additions = patchObject.chunks.flatMap(change => change.changes).filter(change => change.type === 'add')
        additions.forEach(addition => {
          let diffLine = fileLines.find(line => line.line === addition.ln)
          if(diffLine){
            linesToCover++
            fileLinesToCover++
            coveredLines = coveredLines + (diffLine.hit > 0 ? 1 : 0)
            fileCoveredLines = fileCoveredLines + (diffLine.hit > 0 ? 1 : 0)
          }
        })
        if(fileLinesToCover === 0) {
          console.log(`File ${file.file} has no lines to cover`)
        }else {
          console.log(`The coverage percentage ${file.file} is : ` + (fileCoveredLines / fileLinesToCover * 100))
        }
      })
      console.info(`Coverage check is completed.`);
      const overallCov = parseFloat(coveredLines / linesToCover * 100).toFixed(2);
      console.info(`The overall coverage for changed and added lines is ${overallCov}`);
      if(overallCov < minCov ) {
        exitCode = 1
        console.info(`The overall coverage is bellow the minimum of ${minCov}`);
      }
      if(exitCode > 0) {
        console.info(`Something went wrong, check the logs for more info.`);
      } else {
        console.info(`All the checks have passed succesfully.`);
      }
      process.exit(exitCode);
    }).catch(function (err) {
        console.error(err);
        process.exit(1);
    });
  },
  validateOptions: function(options)
  {
    if(!options.project)
    {
      console.log("The project path is mandatory, no project path option passed on -p");
      process.exit(1);
    }
    
  }
}

module.exports = app;
